/*
 * ps300 device support
 */

#include <epicsStdio.h>
#include <devCommonGpib.h>

/******************************************************************************
 *
 * The following define statements are used to declare the names to be used
 * for the dset tables.   
 *
 * A DSET_AI entry must be declared here and referenced in an application
 * database description file even if the device provides no AI records.
 *
 ******************************************************************************/
#define DSET_AI     devAips300
#define DSET_AO     devAops300
#define DSET_BI     devBips300
#define DSET_BO     devBops300
#define DSET_EV     devEvps300
#define DSET_LI     devLips300
#define DSET_LO     devLops300
#define DSET_MBBI   devMbbips300
#define DSET_MBBID  devMbbidps300
#define DSET_MBBO   devMbbops300
#define DSET_MBBOD  devMbbodps300
#define DSET_SI     devSips300
#define DSET_SO     devSops300
#define DSET_WF     devWfps300

#include <devGpib.h> /* must be included after DSET defines */

#define TIMEOUT     1.0    /* I/O must complete within this time */
#define TIMEWINDOW  2.0    /* Wait this long after device timeout */

/******************************************************************************
 * Strings used by the init routines to fill in the znam,onam,...
 * fields in BI and BO record types.
 ******************************************************************************/

static char  *resetList[] = { "Reset","Reset" };
static struct devGpibNames reset = { 2,resetList,0,1 };

static char  *clearList[] = { "Clear","Clear" };
static struct devGpibNames clear = { 2,clearList,0,1 };

static char  *offOnList[] = { "Off","On" };
static struct devGpibNames offOn = { 2,offOnList,0,1 };

static char  *tripModeList[] = { "Manual","Auto" };
static struct devGpibNames   tripMode = { 2,tripModeList,0,1 };

static char  *vsetModeList[] = { "Front Panel","Back Panel" };
static struct devGpibNames   vsetMode = { 2,vsetModeList,0,1 };

/******************************************************************************
 * String arrays for EFAST operations. The last entry must be 0.
 *
 * On input operations,only as many bytes as are found in the string array
 * elements are compared.  Additional bytes are ignored.
 * The first matching string  will be used as a match.
 *
 * For the input operations,the strings are compared literally!  This
 * can cause problems if the instrument is returning things like \r and \n
 * characters.  When defining input strings so you include them as well.
 ******************************************************************************/

static char *hvOffOn[] = {"HVOF", "HVON", 0};

/******************************************************************************
 * Array of structures that define all GPIB messages
 * supported for this type of instrument.
 ******************************************************************************/

static struct gpibCmd gpibCmds[] = {
    /* Param 0 -- Read SCPI identification string */
    {&DSET_SI, GPIBREAD, IB_Q_HIGH, "*IDN?", "%39[^\r\n]", 0, 200, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 1 - SCPI reset command */
    {&DSET_BO, GPIBCMD, IB_Q_HIGH, "*RST", NULL, 0, 80, NULL, 0, 0, NULL, &reset, NULL},

    /* Param 2 - SCPI clear status command */
    {&DSET_BO, GPIBCMD, IB_Q_HIGH, "*CLS", NULL, 0, 80, NULL, 0, 0, NULL, &clear, NULL},

    /* Param 3 - Read SCPI status byte */
    {&DSET_MBBID, GPIBREAD, IB_Q_HIGH, "*STB?", "%d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 4 - Read SCPI event register */
    {&DSET_LI, GPIBREAD, IB_Q_HIGH, "*ESR?", "%d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 5 - Enable SCPI events */
    {&DSET_LO, GPIBWRITE, IB_Q_HIGH, NULL, "*ESE %d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 6 - Read back SCPI enabled events */
    {&DSET_LI, GPIBREAD, IB_Q_HIGH, "*ESE?", "%d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 7 - Enable SCPI service request sources */
    {&DSET_LO, GPIBWRITE, IB_Q_HIGH, NULL, "*SRE %d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 8 - Read back SCPI enabled service request sources */
    {&DSET_LI, GPIBREAD, IB_Q_HIGH, "*SRE?", "%d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 9 - Read SCPI output completion status */
    {&DSET_LI, GPIBREAD, IB_Q_HIGH, "*OPC?", "%d", 0, 80, NULL, 0, 0, NULL, NULL, NULL},

    /* Param 10 - Turn output Off/On */
    {&DSET_BO, GPIBEFASTO, IB_Q_HIGH, 0, NULL, 0, 32, 0, 0, 0, hvOffOn, &offOn, NULL},

    /* Param 11 - Clear voltage/current trip */
    {&DSET_BO, GPIBCMD, IB_Q_HIGH, "TCLR", NULL, 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 12 - Set voltage/current trip reset mode */
    {&DSET_BO, GPIBWRITE, IB_Q_HIGH, NULL, "TMOD%d", 0, 32, 0, 0, 0, NULL, &tripMode, NULL},

    /* Param 13 - Get voltage/current trip reset mode */
    {&DSET_BI, GPIBREAD, IB_Q_HIGH, "TMOD?", "%d", 0, 32, 0, 0, 0, NULL, &tripMode, NULL},

    /* Param 14 - Get VSET mode */
    {&DSET_BI, GPIBREAD, IB_Q_HIGH, "SMOD?", "%d", 0, 32, 0, 0, 0, NULL, &vsetMode, NULL},

    /* Param 15 - Set voltage */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "VSET%.3f", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 16 - Get voltage setting */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "VSET?", "%lf", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 17 - Get voltage reading */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "VOUT?", "%lf", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 18 - Set voltage limit */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "VLIM%.3f", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 19 - Get voltage limit */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "VLIM?", "%lf", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 20 - Set current limit */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "ILIM%.5g", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 21 - Get current limit */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "ILIM?", "%lf", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 22 - Get current reading */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "IOUT?", "%lf", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 23 - Set current trip point */
    {&DSET_AO, GPIBWRITE, IB_Q_HIGH, NULL, "ITRP%.5g", 0, 32, 0, 0, 0, NULL, NULL, NULL},

    /* Param 24 - Get current trip point */
    {&DSET_AI, GPIBREAD, IB_Q_HIGH, "ITRP?", "%lf", 0, 32, 0, 0, 0, NULL, NULL, NULL},

};

/* The following is the number of elements in the command array above.  */
#define NUMPARAMS sizeof(gpibCmds)/sizeof(struct gpibCmd)

/******************************************************************************
 * Initialize device support parameters
 *
 *****************************************************************************/
static long init_ai(int parm)
{
    if(parm==0) {
        devSupParms.name = "devps300";
        devSupParms.gpibCmds = gpibCmds;
        devSupParms.numparams = NUMPARAMS;
        devSupParms.timeout = TIMEOUT;
        devSupParms.timeWindow = TIMEWINDOW;
        devSupParms.respond2Writes = -1;
    }
    return(0);
}
